"""
Secret Rotation Automation System

A comprehensive system for automated secret rotation with encryption,
backup management, and extensible provider/rotator architecture.
"""

from secret_rotator.version import __version__, VERSION

__all__ = [
    "__version__",
    "VERSION",
]

# Package metadata
__author__ = "othaimeen"
__email__ = "othaimeen.dev@gmail.com"
__license__ = "MIT"
